@@ -273,10 +273,8 @@ sub update_v2 ($) {
     my $entry_re = qr/
         # capture the type into $1
         (\w+) \s*
-            # capture all of the text between the brackets into $2
-            {( .*? )}
-            # match the last bracket only if followed by another definition
-            (?= \s* (?:info|program|host|service) \s* { )
+        # capture all of the text between the brackets into $2
+        {( .*? )}
         # capture remaining text (1-2 lines) into $3 for re-processing
         (.*)$
     /xs;
@@ -562,22 +560,23 @@ package Nagios::Service::Status;
 sub status {
     my $self = shift;
     if ( $self->{__parent}{VERSION} > 1.9999999 ) {
-        if ( $self->{current_state} == 1 
-          or $self->{current_state} == 0 ) {
-            return 'PENDING';
+        if ( $self->{current_state} == 0 ) {
+            if ($self->{has_been_checked} == 0) {
+                return 'PENDING';
+            }
+            else {
+                return 'OK';
+            }
         }
-        elsif ( $self->{current_state} == 2 ) {
-            return 'OK';
-        }
-        elsif ( $self->{current_state} == 4 ) {
+        elsif ( $self->{current_state} == 1 ) {
             return 'WARNING';
         }
-        elsif ( $self->{current_state} == 8 ) {
-            return 'UNKNOWN';
-        }
-        elsif ( $self->{current_state} == 16 ) {
+        elsif ( $self->{current_state} == 2 ) {
             return 'CRITICAL';
         }
+        elsif ( $self->{current_state} == 3 ) {
+            return 'UNKNOWN';
+        }
         else {
             return "Unknown Status '$self->{current_state}'";
         }
@@ -593,17 +592,18 @@ package Nagios::Host::Status;
 sub status {
     my $self = shift;
     if ( $self->{__parent}{VERSION} > 1.9999999 ) {
-        if ( $self->{current_state} == 1 
-          or $self->{current_state} == 0 ) {
-            return 'PENDING';
+        if ( $self->{current_state} == 0 ) {
+            if ($self->{has_been_checked} == 0) {
+                return 'PENDING';
+            }
+            else {
+                return 'OK';
+            }
         }
-        elsif ( $self->{current_state} == 2 ) {
-            return 'UP';
-        }
-        elsif ( $self->{current_state} == 4 ) {
+        elsif ( $self->{current_state} == 1 ) {
             return 'DOWN';
         }
-        elsif ( $self->{current_state} == 8 ) {
+        elsif ( $self->{current_state} == 2 ) {
             return 'UNREACHABLE';
         }
         else {
